<?php
namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

use App\Templates;
use App\Exception;

class OrderController {
    
    const FAILOVER_EMAIL = "delivery@teknisa.com";  

    private $orderService;
    private $connection;
    private $util;
    
    public function __construct (\App\Service\OrderService $orderService,
                                 \Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util) {

        $this->orderService       = $orderService;
        $this->connection         = $connection;
        $this->util               = $util;
    }
    
    public function welcome(Request $request, Response $response) {
        
        global $CLIENT_DATA;
        
        $config      = $CLIENT_DATA;
        
        $sql         = "SELECT CDFILIAL FROM FILIAL";
        $result      = $this->connection->fetchAll($sql);
        $versionInfo = $this->getVersionInfo();

        return $response->withJson([
            'client'       => $config['client'],
            'version'      => $versionInfo['version'],
            'revision'     => $versionInfo['revision'],
            'date'         => $versionInfo['date'],
            'filialCount'  => count($result)
        ]);
    }
    
    private function getVersionInfo() {
        
        $file = file_get_contents(__DIR__ . "/../../version.json");
        return json_decode($file, true);
    }
    
    public function getSales(Request $request, Response $response) {
        
        ini_set('memory_limit', '512M');
        $params  = $request->getParsedBody();
        
        $message = date("d-m-Y H:i:s") . ' - Req: getSales | body: ' . json_encode($params);
        $this->util->writeLog('execution', '', $message);
        
        try {

            $this->verifyRefSubsidiaryCode($params);
            $sales = $this->orderService->getSales($params);
            
            return $response->withJson([$sales]);
            
            } catch (\Exception $e) {
                $message = date("d-m-Y H:i:s") . " - " . "Erro na consulta de vendas. " . $e->getMessage();
                $this->util->writeLog('execution', '', $message);
                
                return $response->withJson([
                    'success' => false,
                    'message' => $e->getMessage(),
                    'result'  => []
            ]); 
        }
        
    }
    
    public function getDayReduction(Request $request, Response $response) {
        
        $params  = $request->getParsedBody();
        
        $message = date("d-m-Y H:i:s") . ' - Req: getDayReduction | body: ' . json_encode($params);
        $this->util->writeLog('execution', '', $message);
        
        try {
            
            $this->verifyRefSubsidiaryCode($params);
            $reductions = $this->orderService->getReduction($params);
            
            return $response->withJson([$reductions]);
        
        }catch (\Exception $e) {
            $message = date("d-m-Y H:i:s") . " - " . "Erro na consulta de resumo do dia. " . $e->getMessage();
            $this->util->writeLog('execution', '', $message);
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
        
    }
    
    public function getStock (Request $request, Response $response) {
        
        $params  = $request->getParsedBody();
        
        $message = date("d-m-Y H:i:s") . ' - Req: getStock | body: ' . json_encode($params);
        $this->util->writeLog('execution', '', $message);
        
        try {
            
            $this->verifyRefSubsidiaryCode($params);
            $stock = $this->orderService->getStockExit($params);
            
            return $response->withJson([$stock]);
        
        }catch (\Exception $e) {
            $message = date("d-m-Y H:i:s") . " - " . "Erro na consulta de movimentação de estoque. " . $e->getMessage();
            $this->util->writeLog('execution', '', $message);
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }    
    }
    
    public function getDayCloses(Request $request, Response $response) {
        
        $params  = $request->getParsedBody();
        
        $message = date("d-m-Y H:i:s") . ' - Req: getDayCloses | body: ' . json_encode($params);
        $this->util->writeLog('execution', '', $message);
        
        try {
            
            $this->verifyRefSubsidiaryCode($params);
            $dayCloses = $this->orderService->getDayCloses($params);
            
            return $response->withJson([$dayCloses]);
        
        }catch (\Exception $e) {
            $message = date("d-m-Y H:i:s") . " - " . "Erro na consulta de fechamentos de caixa. " . $e->getMessage();
            $this->util->writeLog('execution', '', $message);
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }    
    }
    
    public function getClosingSummary(Request $request, Response $response) {
        
        $params  = $request->getParsedBody();
        
        $message = date("d-m-Y H:i:s") . ' - Req: getClosingSummary | body: ' . json_encode($params);
        $this->util->writeLog('execution', '', $message);
        
        try {
            
            $this->verifyRefSubsidiaryCode($params);
            $dayCloses = $this->orderService->getClosingSummary($params);
            
            return $response->withJson([$dayCloses]);
            return $response->withJson([0]);
        
        }catch (\Exception $e) {
            $message = date("d-m-Y H:i:s") . " - " . "Erro na consulta de resumo de fechamentos do caixa. " . $e->getMessage();
            $this->util->writeLog('execution', '', $message);
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }    
    }
    
    private function verifyRefSubsidiaryCode ($params) {
        if(!isset($params['subsidiaryCode']) || $params['subsidiaryCode'] === null) {
            if(!isset($params['subsidiaryExtCode']) || $params['subsidiaryExtCode'] === null)
                throw new Exception \ParamsException("O campo subsidiaryCode é obrigatório. Favor enviar o código da filial na requisição.");
        }
    }
    
    public function importSale(Request $request, Response $response) {
        ini_set('memory_limit', '512M');
        
        $params  = $request->getParsedBody();
        
        $sales = [];
        $multipleImports = isset($params[0]);
        
        if($multipleImports) {
            $sales = $params;
        } else {
            array_push($sales, $params);
        }
        
        $saleData = '';
        
        $this->connection->beginTransaction();
        
        try {
            
            $idSubProjeto = $this->orderService->getIdSubProjeto();
            
            if(empty($idSubProjeto)) {
                $this->orderService->registerIdSubProjeto();
            }
            
            foreach($sales as $sale) { 
                
                if(empty($sale)) {
                    throw new Exception\ParamsException("Nenhum parametro recebido.");
                }
                
                $subsidiaryCode    = isset($params['subsidiaryCode']) ? $params['subsidiaryCode'] : null;
                $subsidiaryExtCode = isset($params['subsidiaryExtCode']) ? $params['subsidiaryExtCode'] : null;
                
                if(empty($subsidiaryCode) && empty($subsidiaryExtCode)) {
                    throw new Exception\ParamsException("Nenhuma Filial foi enviada na consulta. Revise os parametros enviados.");
                }
                
                $generalParams = $this->orderService->getBranchDetails($subsidiaryCode, $subsidiaryExtCode);
                $fileName = "{$generalParams['CDFILIAL']}-{$sale['noteSeries']}-{$sale['cupomNumber']}";
                $saleData = "Cupom: {$sale['cupomNumber']} | Filial: {$subsidiaryCode} | Data da venda: {$sale['saleDate']}";
                
                $this->util->writeLog('execution', '', date("d-m-Y H:i:s") . " - " . "Iniciando processo de importação da venda: {$saleData}.");
                
                $cupomNumber = $generalParams['IDTPEMISSAOFOS'] == 'FNC'? str_pad($sale['cupomNumber'], 10, "0", STR_PAD_LEFT) : str_pad($sale['cupomNumber'], 6, "0", STR_PAD_LEFT);
                
                $getDuplicated = $this->orderService->validateDuplicatedSale($generalParams['CDFILIAL'], $cupomNumber, $sale['chaveSefaz']);
                
                if(count($getDuplicated) > 0) {
                    throw new Exception\ParamsException("Cupom {$cupomNumber} - chaveAcesso {$chaveSefaz} já importado anteriormente para a filial {$cdFilial}.");
                }
                
                $insertSubsidizedConsumption =  'N';
                
                if(isset($sale['origin'])) {
                    if ($sale['origin'] == 'online') {
                        $insertSubsidizedConsumption = 'S';
                    }
                }
        
                $this->orderService->processSalesImport($sale, $subsidiaryCode, $generalParams, $insertSubsidizedConsumption, $idSubProjeto);
            }
            
            $this->connection->commit();
            $this->util->writeLog('execution', '', date("d-m-Y H:i:s") . " - " . "Venda: {$saleData} importada com sucesso.");
            return $response->withJson([
                'success' => true,
                'message' => "Venda importada com sucesso.",
                'result'  => []]);
        
        }catch (\Exception $e) {
            $fileName = isset($fileName) ? $fileName : "INDEFINIDO";
            $this->util->writeLog('error', $fileName, "Erro na importação. {$saleData}.\n". $e->getMessage());
            $this->util->writeLog('execution', '', date("d-m-Y H:i:s") . " - " . "Erro na importação de venda. {$saleData}.\n" . date("d-m-Y H:i:s") . " - " . $e->getMessage());
            return $response->withJson([
                'success' => false,
                'message' => "Erro na importação. {$saleData} <br>".$e->getMessage(),
                'result'  => []
            ]); 
        }    
    }
    
    public function cancelSale(Request $request, Response $response) {
        ini_set('memory_limit', '512M');
        
        $params  = $request->getParsedBody();
        $saleData = '';
        
        $this->connection->beginTransaction();
        
        try {

            if(empty($params)) {
                throw new Exception\ParamsException("Nenhum parametro recebido.");
            }
            
            $subsidiaryCode    = isset($params['subsidiaryCode']) ? $params['subsidiaryCode'] : null;
            $subsidiaryExtCode = isset($params['subsidiaryExtCode']) ? $params['subsidiaryExtCode'] : null;
            
            if(empty($subsidiaryCode) && empty($subsidiaryExtCode)) {
                throw new Exception\ParamsException("Nenhuma Filial foi enviada na consulta. Revise os parametros enviados.");
            }
            
            if(!isset($params['chaveSefaz']) || empty($params['chaveSefaz'])) {
                throw new Exception\ParamsException("Chave de acesso da venda não informado.");
            }
            
            if(!isset($params['saleDate']) || empty($params['saleDate'])) {
                throw new Exception\ParamsException("Data da venda não informado.");
            }
            
            $generalParams = $this->orderService->getBranchDetails($subsidiaryCode, $subsidiaryExtCode);
            $fileName = "{$generalParams['CDFILIAL']}-{$params['chaveSefaz']}";
            $saleData = "ChaveAcesso: {$params['chaveSefaz']} | Filial: {$subsidiaryCode} | Data da venda: {$params['saleDate']}";
            $this->util->writeLog('execution', '', date("d-m-Y H:i:s") . " - " . "Iniciando processo de cancelamento de venda importada: {$saleData}.");
            
            $this->orderService->processCancelSale($params, $subsidiaryCode, $generalParams);
            
            $this->connection->commit();
            $this->util->writeLog('execution', '', date("d-m-Y H:i:s") . " - " . "Venda: {$saleData} cancelada com sucesso.");
            return $response->withJson([
                'success' => true,
                'message' => "Cancelamento realizado com sucesso.",
                'result'  => []
            ]);
        
        }catch (\Exception $e) {
            $fileName = isset($fileName) ? $fileName : "INDEFINIDO";
            $this->util->writeLog('error', $fileName, "Erro na importação. {$saleData}.\n". $e->getMessage());
            $this->util->writeLog('execution', '', date("d-m-Y H:i:s") . " - " . "Erro no cancelamento de venda. {$saleData}.\n" . date("d-m-Y H:i:s") . " - " . $e->getMessage());
            return $response->withJson([
                'success' => false,
                'message' => "Erro no cancelamento. {$saleData} <br>".$e->getMessage(),
                'result'  => []
            ]); 
        }    
    }
    
    private function checkBodyResquest($request, $type) {
        
        $requiredFieldsSale = [
            'noteNumber',
            'noteSeries',
            'chaveSefaz',
            'protocSefaz',
            'cupomNumber',
            'cashierOpeningDate',
            'emissionDate',
            'saleDate',
            'saleStatus',
            'noteStatus',
            'protoCancSefaz',
            'saleTotalAmount',
            'saleSubsidizedAmount',
            'noteTotalAmount',
            'totalDiscountAmount',
            'totalAmountAddition',
            'totalAmountPIS',
            'totalAmountCOFINS',
            'totalAmountICMS',
            'payments',
            'itens',
            
        ];
        
        $requiredFieldsTaxes = [
            'productCode',
            'productName',
            'taxCode',
            'aliquot',
            'icmsReductionPercentage',
            'icmsCalculationBasis',
            'incidencePisCofins',
            'aliqIBPTF',
            'aliqIBPTE',
            'cfopCode',
            'cstIcms',
            'cstPisCofins',
            'pisTaxRate',
            'cofinsTaxRate',
            'fcpTaxRate',
            'effectiveIcmsTaxRate',
            'effectiveIcmsRedPercentage',
            'icmsExemptionRate',
            'fiscalBenefitCode',
        ];
        
        if($type === 'sale') {
            foreach($body as $key => $value) {
                if (!in_array($key, $requiredFieldsSale)) {
                    throw new Exception\ParamsException("É obrigatório o envio do campo {$key} no payload. Revise os parametros enviados");
                }
            }
        }
        
        if($type === 'tax') {
            foreach($request as $body) {
                foreach($body as $key => $value) {
                    if (!in_array($key, $requiredFieldsTaxes)) {
                       throw new Exception\ParamsException("É obrigatório o envio do campo {$key} no payload. Revise os parametros enviados");
                    }
                }
            }
        }
    }
    
    public function getProductTaxes(Request $request, Response $response) {
         
        $params  = $request->getParsedBody();
        
        $message = date("d-m-Y H:i:s") . ' - Req: getProductTaxes | body: ' . json_encode($params);
        $this->util->writeLog('execution', '', $message);
         
        try {
            
            $this->verifyRefSubsidiaryCode($params);
            if(count($params['productsCode']) > 100)
                throw new Exception\ParamsException("Consulta limitada a 100 registros."); 
                
            $taxes = $this->orderService->getProductTaxes($params);
            
            return $response->withJson([$taxes]);
        
        }catch (\Exception $e) {
            $message = date("d-m-Y H:i:s") . " - " . "Erro na consulta de impostos do produto. " . $e->getMessage();
            $this->util->writeLog('execution', '', $message);
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }    
    }
    
    public function setProductTaxes(Request $request, Response $response) {
         $params  = $request->getParsedBody();
         
        try {
            
            if(!isset($params['taxes']) || empty($params['taxes'])) {
                
            }
            
            $this->checkBodyResquest($params['taxes'], 'tax');
            $this->verifyRefSubsidiaryCode($params);

            if(count($params['taxes']) > 100)
                throw new Exception\ParamsException("Importação limitada a 100 registros."); 
                
            $this->orderService->setProductTaxes($params);
            
            return $response->withJson([
                'success' => true,
                'message' => "Imposto(s) cadastrado com sucesso.",
                'result'  => []]);
        
        }catch (\Exception $e) {
            $message = date("d-m-Y H:i:s") . " - " . "Erro no cadastro de impostos do produto. " . $e->getMessage();
            $this->util->writeLog('execution', '', $message);
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }    
    }
    
    public function getSaleProducts(Request $request, Response $response) {
         ini_set('memory_limit', '512M');
        
        $params  = $request->getParsedBody();
        
        $message = date("d-m-Y H:i:s") . ' - Req: getProductTaxes | body: ' . json_encode($params);
        $this->util->writeLog('execution', '', $message);
        
        $paramsRequest  = $request->getParams();
        
        $page = isset($paramsRequest['page']) ? $paramsRequest['page'] : 1;
        $perPage = isset($paramsRequest['perPage']) ? $paramsRequest['perPage'] : 300;
        
        try {

            $this->verifyRefSubsidiaryCode($params);
            $return = $this->orderService->getsaleProducts($params, $page, $perPage);
            
            return $response->withJson($return);
            
        } catch (\Exception $e) {
                $message = date("d-m-Y H:i:s") . " - " . "Erro na consulta de produtos. " . $e->getMessage();
                $this->util->writeLog('execution', '', $message);
                
                return $response->withJson([
                    'success' => false,
                    'message' => $e->getMessage(),
                    'result'  => []
                ]); 
        }
    }
}
